/* COPYRIGHT (C) GUPTA TECHNOLOGIES, INC. 1984-1992 */
/*
FUNCTION NAME
  test - C Program Example
REVISION HISTORY
  03/13/90 GTI original
  09/20/92 DBS correct declaration of main
EXTERNAL INTERFACE
  FUNCTION
    Main function of the test program.
  DESCRIPTION
    The is a sample program to demonstrate some of the interfacse to SQLBASE
  RETURN
INTERNAL DESCRIPTION
  The cursor is cleared.  The database is connected to by calling  sqlcon  and
  passing the database name string.  The table is created  by  calling	sqlcex
  with the CREATE TABLE string.  The INSERT command  is  compiled.   The  DATA
  file is opened.  Lines are read from the  DATA  file.   For  each  line  the
  commas in the line are located to separate  each  field  value.   Sqlbnn  is
  called to bind each field value to the appropriate bind variable.  After the
  input line has been scanned and the data has been bound, sqlexe is called to
  execute the INSERT command to insert the row/data into the  database.   This
  is done until the end of the DATA file is reached.  The index is created  by
  calling sqlcex with the CREATE INDEX string.	The SELECT command is compiled
  by calling sqlcom.  For each column in the SELECT command SQLDES  is	called
  to get descriptive information.  Once the descriptive information is gotten,
  sqlssb is called to set  the	buffer	of  where  the	select	data  will  be
  returned.  The SELECT command is  executed  by  calling  sqlexe.   Rows  are
  fetched from the SELECT command until the end of the SELECT is reached.  The
  is done by repeatedly calling sqlfet.  Each row is printed to  the  standard
  output.  The UPDATE command is compiled and executed by calling sqlcex  with
  the UPDATE command string.  The database is  disconnected  from  by  calling
  sqldis.  Control is returned to the system.
*/
#include "sql.h"
#include "stdio.h"
#include "string.h"

	  SQLTCUR   cur = 0;		/* SQLBASE cursor number	    */
	  SQLTRCD   rcd = 0;		/* SQLBASE return code		    */

void	  main(argc, argv)
	  int	    argc;		/* argument count		    */
	  char**    argv;		/* -> argument vector		    */
{
	  char*     p;			/* misc. pointer		    */
	  FILE*     in; 		/* input file			    */
	  SQLTDAP   cp; 		/* character pointer		    */
	  SQLTDAL   length;		/* length			    */
	  SQLTPDL   pdl;		/* program buffer length	    */
	  int	    rows;		/* number of rows		    */

	  SQLTDDT   ddt;		/* database data type		    */
	  SQLTPDT   pdt;		/* program data type		    */
	  SQLTBNN   bnn;		/* bind number			    */
	  SQLTSLC   slc;		/* select list column		    */
	  SQLTNBV   nbv;		/* number of bind variables	    */
	  SQLTNSI   nsi;		/* number of select items	    */
	  SQLTRCD   error;		/* error number 		    */

	  char	    line[200];		/* I/O line			    */
static	  char	    dbnam[] = "demo";	/* database name		    */
static	  char	    ctbcom[] =		/* CREATE TABLE command 	    */
"CREATE TABLE X (A NUMBER, B DATETIME, C CHAR(30), D NUMBER)";

static	  char	    inscom[] =		/* INSERT command		    */
"INSERT INTO X(A, B, C, D) VALUES (:1, :2, :3, :4)";

static	  char	    ctincom[] = 	/* CREATE INDEX command 	    */
"CREATE INDEX A ON X(A)";

static	  char	    updcom[] =		/* UPDATE command		    */
"UPDATE X SET A = '9999'";

static	  char	    selcom1[] = 	/* SELECT command		    */
"SELECT A, D, C FROM X";

static	  char	    selcom2[] = 	/* SELECT command		    */
"SELECT C FROM X";
  /*
    CONNECT TO THE DATABASE
  */

  cur = 0;
  if (rcd = sqlcnc(&cur, dbnam, 0))	/* perform connect operation	    */
    failure("CONNECT");

  /*
    CREATE THE TABLE
  */

  if (sqlcex(cur, ctbcom, 0))		/* failure on compile and execute?  */
    failure("CREATE TABLE");

  /*
    COMPILE THE INSERT COMMAND
  */

  if (sqlcom(cur, inscom, 0))		/* failure on compile?		    */
    failure("INSERT COMPILE");

  /*
    INSERT THE DATA
  */

  if (!(in = fopen("DATA", "r")))	/* failure on open of file?	    */
  {
    printf("tst: can't open data file\n");
    exit(1);
  }

  rows = 0;				/* initialize number of rows	    */
  pdt = SQLPBUF;			/* set program data type of buffer  */

  while (fgets(line, sizeof(line), in)) /* scan the input		    */
  {
    rows++;				/* increment number of rows	    */
    line[strlen(line) - 1] = 0; 	/* remove new line		    */

    if (sqlnbv(cur, &nbv))		/* failure on nbv?		    */
      failure("GET NUMBER OF BIND VARIABLES");

    for (cp = line, bnn = 1;		/* scan the line		    */
	 bnn <= nbv; bnn++)
    {
      length = (p = strchr(cp, ',')) ?	/* locate comma 		    */
	  p - cp : strlen(cp);
      sqlbnn(cur, bnn, cp, length, 0,	/* bind by number		    */
	  pdt);
      cp += length;			/* locate end			    */
      if (*cp == ',')			/* comma?			    */
	cp++;
    }

    if (sqlexe(cur))			/* failure on execute?		    */
    {
      printf("%d ROWS INSERTED\n", rows);
      failure("INSERT EXECUTE");
    }
  }

  /*
    CREATE THE INDEX
  */

  if (sqlcex(cur, ctincom, 0))		/* failure on create index?	    */
    failure("CREATE INDEX");

  /*
    COMPILE THE SELECT COMMAND
  */

  if (sqlcom(cur, selcom1, 0))		/* failure on select compile?	    */
    failure("SELECT COMPILE");

  /*
    GET DESCRIPTIVE INFORMATION ABOUT SELECT
  */

  cp = line;				/* set pointer to input line	    */

  if (sqlnsi(cur, &nsi))		/* get # select items		    */
    failure("GET NUMBER OF SELECT ITEMS");

  for (slc = 1; slc <= nsi; slc++)	/* get information on each column   */
  {
    if (sqldes(cur, slc, &ddt, &pdl,	/* failure on describe? 	    */
	SQLNPTR, SQLNPTR, SQLNPTR, SQLNPTR))
      failure("SELECT DESCRIBE");
    if (sqlssb(cur, slc, pdt, cp,	/* set select buffer		    */
	pdl, 0, SQLNPTR, SQLNPTR))
      failure("SET SELECT BUFFER");
    cp += (pdl + 1);			/* locate next area		    */
  }

  /*
    EXECUTE THE SELECT COMMAND
  */

  if (sqlexe(cur))			/* failure on select execute?	    */
    failure("SELECT EXECUTE");

  /*
    FETCH AND DISPLAY THE DATA
  */

  length = cp - line;			/* compute the length		    */
  *cp = 0;				/* concatenate a zero to the string */

  for (;;)
  {
    memset(line, ' ', length);		/* fill the line with spaces	    */

    if (rcd = sqlfet(cur))		/* failure or end of file?	    */
      break;

    printf("%s\n", line);		/* print the line		    */
  }

  if (rcd != 1) 			/* failure on fetch		    */
    failure("FETCH");

  /*
    UPDATE THE DATA
  */

  if (sqlcex(cur, updcom, 0))		/* failure on update?		    */
    failure("UPDATE");

  /*
    DISCONNECT FROM THE DATABASE
  */

  if (rcd = sqldis(cur))		/* failure on disconnect?	    */
  {
    cur = 0;
    failure("DISCONNECT");
  }
}

	  failure(p)
	  char*     p;			/* pointer to string		    */
{
	  SQLTEPO   epo;		/* error position		    */
  if (cur)				/* cursor connected?		    */
  {
    sqlrcd(cur, &rcd);			/* get the error		    */
    sqlepo(cur, &epo);			/* get error position		    */
    sqldis(cur);
  }
  printf("TEST: failure on %s rcd=%d, epo=%d\n",
      p, rcd, epo);
  exit(1);
}
